@@ -25,9 +25,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 /* global prototype definitions */
 extern char *safeban(struct channel *, char *, char *);
-extern char *getlamer(struct channel *);
 extern char *getfirsthostless(struct channel *, char *);
-extern char *isnickbk(struct channel *, char *);
 extern char *getkickreason(char *);
 extern int topiccount(struct channel *);
 extern int settopic(struct channel *);
@@ -55,6 +53,8 @@ extern int isservice(char *);
 extern int istrusted(char *);
 extern int isme(char *);
 extern int isop(struct channel *, char *);
+extern int isvoice(struct channel *, char *);
+extern int isvoicenow(struct channel *, char *);
 extern int isvalidlogon(struct channel *, char *);
 extern int isopa(char *);
 extern int ishostless(char *);
@@ -67,9 +67,7 @@ extern int isonchan(struct channel *, char *);
 extern int issameuser(struct channel *, char *, char *);
 extern int isopnow(struct channel *, char *);
 extern int nickchange(struct channel *, char *, char *);
-extern int iswordbk(struct channel *, char *);
 extern int isflood(struct channel *, char *, char *, char *);
-extern int isvoicenow(struct channel *, char *);
 extern int haschanflags(struct channel *, char*);
 extern int hasadminflags(char *, char *);
 extern int ispermban(struct channel *, char *);
@@ -82,6 +80,7 @@ extern int whichctcp(char *);
 extern int whichcommand(char *, int);
 extern int gettobek(struct channel *, char *);
 extern int checkforansi(struct channel *, char *, char *, char *);
+extern int checkfornickflood(struct channel *, char *, char *);
 extern int checkforflood(struct channel *, char *, char *, char *);
 extern int checkforbadword(struct channel *, char *, char *, char *);
 extern int checkforbadnick(struct channel *, char *, char *);
@@ -97,7 +96,7 @@ extern int delusermask(struct channel *, char *, char *);
 extern int deltrusted(char *);
 extern int delservice(char *);
 extern int delchannel(struct channel *);
-extern int deladverts(struct channel *, char *);
+extern int deladverts(struct channel *, char *, int);
 extern int getondiskmsgcount(struct channel *, char *);
 extern int getondiskjoincount(struct channel *, char *);
 extern int getrandommsg(struct channel *, char *, int, char *, int, int);
@@ -108,7 +107,7 @@ extern int logoff(struct channel *, char *);
 extern int logon(struct channel *, struct botuser *, char *, char *, int);
 extern int timed_cmp(struct timed *, struct timed *);
 extern int isfriend(struct channel *, char *, char *);
-extern int deltachanflags(struct channel *, char *, char);
+extern int deltaflags(struct channel *, char *, char);
 extern int isvalidchannel(char *);
 extern int hasuserflags(struct channel *, char *, char *, char *);
 extern unsigned int getrandom(unsigned int);
@@ -125,14 +124,17 @@ extern struct server *addnewserver(char *, int, int, int, int, int, char *);
 extern struct server *getserver(void);
 extern struct advert *addnewadvert(struct channel *, char *, char *);
 extern struct maskstruct *addnewmask(char *);
-extern struct maskstruct *addnewnickbk(struct channel *, char *, char *);
-extern struct maskstruct *addnewwordbk(struct channel *, char *, char *);
+extern struct maskstruct *addnewnickbk(struct channel *, char *, char *, char *);
+extern struct nickchange *addnewnickchange(struct channel *, char *);
+extern struct maskstruct *addnewwordbk(struct channel *, char *, char *, char *);
 extern struct maskstruct *addnewtrusted(char *);
 extern struct maskstruct *addnewservice(char *);
 extern struct maskstruct *addnewkickreason(char *);
 extern struct maskstruct *addnewusermask(struct channel *, char *, char *);
 extern struct maskstruct *addnewadminmask(char *, char *);
 extern struct maskstruct *editmask(struct maskstruct *, char *, char *);
+extern struct maskstruct *isnickbk(struct channel *, char *);
+extern struct maskstruct *iswordbk(struct channel *, char *);
 extern struct pubmsg *addnewpubmsg(struct channel *, time_t, char *, char *, char *);
 extern struct part *addnewpart(struct channel *, time_t, char *, char *);
 extern struct join *addnewjoin(struct channel *, time_t, char *, char *);
@@ -145,6 +147,7 @@ extern time_t lastoff(struct channel *, char *, char *);
 extern void sendadverts(struct channel *, char *, char *);
 extern void resetparts(struct channel *, char *, char *);
 extern void initpubmsgs(struct channel *);
+extern void freenickchanges(struct channel *, char *);
 extern void freepubmsguser(struct channel *, char *, char *);
 extern void lockchan(struct channel *, char *, int, char *, char *);
 extern void unlockchan(struct channel *, char *, char *);
@@ -165,6 +168,7 @@ extern void banuser(struct channel *, time_t, char *);
 extern void unbanuser(struct channel *, time_t, char *);
 extern void kickuser(struct channel *, time_t, char *, char *);
 extern void sendreply(char *, int, int, int, char *);
+extern void sendwall(char *, struct channel *, int, char *);
 extern void sendchannelnotice(struct channel *, int, char *);
 extern void addcommand(struct channel *, char *, int, char *, char *, char *);
 extern void delcommand(struct channel *, char *, int, char *, char *, char *);
@@ -175,6 +179,7 @@ extern void logoncommand(struct channel *, char *, int, char *, char *, char *);
 extern void changetobek(struct channel *, char *, int);
 extern void setnick(void);
 extern void setumode(void);
+extern void sendqauth(void);
 extern void syncvoices(struct channel *);
 extern void syncops(struct channel *);
 extern void syncpermbans(struct channel *);
@@ -190,6 +195,7 @@ extern void updateseen(struct channel *, char *, char *);
 extern void showlast(struct channel *, char *, int, char *);
 extern void showseen(struct channel *, char *, int, char *);
 extern void setchanmode(struct channel *);
+extern void setchankey(struct channel *, char *);
 extern void channelsync(struct channel *);
 extern void cleartobek(struct channel *);
 extern void getnthmode(char *, int, char *);
@@ -198,7 +204,7 @@ extern void quit(void);
 extern void parsectcp(char *, char *, int, char *);
 extern void nstats(struct channel *, char *, int, char *);
 extern void wall(struct channel *, char *, char *, char *);
-extern void cleanautheds(struct channel *);
+extern void cleanautheds(struct channel *, int);
 extern void listchanusers(void);
 extern void timed_del(struct timed *);
 extern void mmode_del(struct channel *, struct mmode *);
