@@ -175,12 +175,12 @@ int wordcount(char *line)
 	int ppos;
 	int nppos;
 	int words;
-	char word[STRINGLEN + 1];
+	char word[WRITE_BUFFER_LENGTH + 1];
 
 	ppos = 0;
 	words = 0;
 	while(1) {
-		nppos = parse(line, ppos, " ", word, STRINGLEN, 0);
+		nppos = parse(line, ppos, " ", word, WRITE_BUFFER_LENGTH, 0);
 		if(nppos == ppos || nppos < 0) {
 			break;
 		}
@@ -449,7 +449,8 @@ int parse(char *from, int pos, char *delim, char *to, int to_length, int fixed)
         if(k >= to_length) {
 		/* insufficient buffer space */
 #ifdef DEBUG
-                oer_debug(OER_DEBUG_WARNING, "parse->destination buffer too small! E-mail equ@equnet.org for a fix\n");
+                oer_debug(OER_DEBUG_WARNING, "parse->source length at least %d, destination length %d\n", k, to_length);
+                oer_debug(OER_DEBUG_WARNING, "parse->destination buffer too small!\n");
 #endif
                 return -1;
         }
@@ -529,26 +530,34 @@ char *mysqlsafestr(char *from, char *to, int to_len)
 	int i;
 	int k;
         int len;
+	int bailout;
 
-	for(i = 0, k = 0, len = strlen(from); i < len && k < to_len; i++) {
-		/* these need doubling, we don't need to check for " because
-		 all our strings are done with hyphens */
+	for(i = 0, k = 0, len = strlen(from), bailout = 0; i < len && k < to_len; i++) {
 		if(from[i] == '\'' || from[i] == '\\') {
-			if((k + 1) < to_len) {
-				to[k++] = from[i];
-				to[k++] = from[i];
-				continue;
-			} else {
-				return NULL;
+			if(k + 1 >= to_len) {
+				/* bail out */
+				bailout = 2;
+				break;
 			}
+			to[k++] = from[i];
+			to[k++] = from[i];
+		} else {
+			if(k >= to_len) {
+				/* bail out */
+				bailout = 1;
+				break;
+			}
+			to[k++] = from[i];
 		}
-		/* normal char, copy */
-		to[k++] = from[i];
         }
-	if(k >= to_len) {
-		return NULL;
+#ifdef DEBUG
+	if(bailout == 1 || bailout == 2) {
+		/* insufficient buffer space */
+                oer_debug(OER_DEBUG_WARNING, "mysqlsafestr->source length at least %d, destination length %d\n", k + bailout, to_len);
+                oer_debug(OER_DEBUG_WARNING, "mysqlsafestr->destination buffer too small! E-mail equ@equnet.org for a fix\n");
 	}
-        to[k] = '\0';
+#endif
+	to[k] = '\0';
 	return to;
 }
 
