@@ -80,6 +80,8 @@ enum NTHMODE_RETURN_VALUES {
 	OER_NTHMODE_DVOICE,
 	OER_NTHMODE_BAN,
 	OER_NTHMODE_UNBAN,
+	OER_NTHMODE_CHANMODE_WITHOUT_PARAMS,
+	OER_NTHMODE_CHANMODE_WITH_PARAMS,
 	OER_NTHMODE_UNKNOWN
 };
 
@@ -186,19 +188,12 @@ enum SERVERCONNECTION_RETURN_VALUES {
 };
 
 enum REGISTERCONNECTION_RETURN_VALUES {
-	OER_REGISTERCONNECTION_ERR_GETPWUID,
-	OER_REGISTERCONNECTION_ERR_WRITE,
-	OER_REGISTERCONNECTION_ERR_HANDLESERVERDATA,
-	OER_REGISTERCONNECTION_ALREADY_REGISTERED,
-	OER_REGISTERCONNECTION_EVERYTHING_OK
-};
-
-enum REGISTERCONNECTION_STATUS {
-	OER_REGISTERCONNECTION_STATUS_BEGIN,
-	OER_REGISTERCONNECTION_STATUS_NICK,
-	OER_REGISTERCONNECTION_STATUS_USER,
-	OER_REGISTERCONNECTION_STATUS_DONE,
-	OER_REGISTERCONNECTION_STATUS_ERROR
+        OER_REGISTERCONNECTION_ERR_GETPWUID,
+        OER_REGISTERCONNECTION_ERR_WRITE,
+        OER_REGISTERCONNECTION_ERR_HANDLESERVERDATA,
+        OER_REGISTERCONNECTION_ALREADY_REGISTERED,
+        OER_REGISTERCONNECTION_ERROR,
+        OER_REGISTERCONNECTION_EVERYTHING_OK
 };
 
 enum HANDLESERVERDATA_RETURN_VALUES {
@@ -231,11 +226,14 @@ enum OER_BAN_TYPES {
 enum OER_TIMED_TYPES {
         OER_TIMED_TYPE_NORMAL = 1,
         OER_TIMED_TYPE_KICK,
+        OER_TIMED_TYPE_USERHOST,
         OER_TIMED_TYPE_INVALID
 };
 
 enum OER_TIMED_PRIORITIES {
-        OER_TIMED_PRIORITY_FLOOD = 1,
+        /* lowest priority first */
+        OER_TIMED_PRIORITY_WALL = 1,
+        OER_TIMED_PRIORITY_FLOOD,
         OER_TIMED_PRIORITY_NOTICE,
         OER_TIMED_PRIORITY_PRIVMSG,
         OER_TIMED_PRIORITY_NORMAL,
@@ -256,12 +254,10 @@ enum OER_TIMED_PRIORITIES {
 #define OER_ACTIONS_PER_MAINLOOP 2
 #define OER_KICKS_PER_MAINLOOP 1
 #define OER_KICK_INTERVAL 2
+#define OER_GN_INTERVAL 10
 #define OER_NICKS_PER_USERHOST 5
 /* multiple kicks aren't supported for backwards compatibility */
 #define OER_NICKS_PER_KICK 1
-#define OER_MAXTARGETS 20
-#define OER_TARGET_DELAY 120
-#define OER_WALL_DELAY 2
 #define OER_DEFAULT_DELIM "::"
 #define OER_DEFAULT_DELIM_LEN 2
 #define OER_QUOTE_MIN_LENGTH 10
@@ -281,6 +277,7 @@ enum OER_TIMED_PRIORITIES {
 #define OER_RECONNECT_DELAY 30
 #define OER_NETJOIN_DELAY 15
 #define OER_PUBMSGS 20
+#define OER_NICKCHANGES 20
 #define OER_SERVER_IS_STONED_FOR 300
 #define OER_LINENOISE_INTERVAL_MIN 1
 #define OER_MASSMODE_LIMIT 3
@@ -291,6 +288,8 @@ enum OER_TIMED_PRIORITIES {
 #define OER_SEEN_MAX 10
 #define OER_TOPICS 20
 #define OER_TOPICLEN 160
+#define TOPIC_SEPARATOR " | "
+#define CONNECTION_STATE_PREFIXES "^~+=-"
 #define OER_PERMBANS 25
 #define OER_LOGON_TIMEOUT 1800
 #define OER_BANVARS_AUTO_REJOIN 30
@@ -309,6 +308,8 @@ enum OER_TIMED_PRIORITIES {
 #define PUBMSG_FLOOD_INTERVAL 60
 #define PUBMSG_FLOOD_REPEAT 3
 #define PUBMSG_FLOOD_REPEAT_EXPIRE 1800
+#define NICKFLOOD_EXPIRE 7200
+#define NICKFLOOD_CHANGES 5
 #define TXT_BUFFER 1024
 /* this is the ircd buffer size */
 #define IRCD_BUFFER_LENGTH 512
@@ -332,10 +333,12 @@ enum OER_TIMED_PRIORITIES {
 #define OER_MAX_LIST_LINES 5
 #define OER_HELP "For a complete command reference go to http://oer.equnet.org/help.html"
 #define DEFAULT_NICKBK_MESSAGE "prohibited nick"
+#define DEFAULT_WORDBK_MESSAGE "word ban-kick triggered, have a nice life..."
 #define DBSTATUS_MSG "At least one of the executed database queries failed, more \
 details possibly available on the oer+MySQL console"
 #define OER_REALNAME "http://oer.equnet.org/help.html"
-#define OER_COPYRIGHT1 "oer+MySQL version 1.0-24, Copyright (C) 2000-2001 EQU <equ@equnet.org>"
+#define OER_UNKNOWN "unknown"
+#define OER_COPYRIGHT1 "oer+MySQL version 1.0-25, Copyright (C) 2000-2001 EQU <equ@equnet.org>"
 #define OER_COPYRIGHT2 "oer+MySQL comes with ABSOLUTELY NO WARRANTY; for details see the\n\
 GNU General Public License (COPYING) provided with this distribution.\n\
 This is free software, and you are welcome to redistribute it under\n\
@@ -355,6 +358,8 @@ struct floodvars {
 	int interval;
 	int lines;
 	int chars;
+	int nickflood_expire;
+	int nickflood_changes;
 };
 
 struct banvars {
@@ -377,6 +382,14 @@ struct locked {
 	char host[USERHOSTLEN + 1];
 };
 
+struct qauth {
+	int hasauth;
+	int authed;
+	char q[USERHOSTLEN + 1];
+	char name[NICKLEN + 1];
+	char password[TINYSTRINGLEN + 1];
+};
+
 struct chanuser {
 	char *nick;
 	char *userhost;
@@ -416,6 +429,13 @@ struct pubmsg {
 	struct pubmsg *next;
 };
 
+struct nickchange {
+        time_t at;
+        char *userhost;
+        struct nickchange *prev;
+        struct nickchange *next;
+};
+
 struct join {
 	time_t at;
 	char *nick;
@@ -453,7 +473,8 @@ struct timed {
 
 struct maskstruct {
 	char *mask;
-	char *optstring;
+	char *optstring1;
+	char *optstring2;
 	struct maskstruct *prev;
 	struct maskstruct *next;
 };
@@ -465,6 +486,19 @@ struct advert {
         struct advert *next;
 };
 
+struct registered {
+        int begin;
+        int pass;
+        int nick;
+        int user;
+        int done;
+};
+
+struct last_action {
+        time_t ts;
+        unsigned int barrier;
+};
+
 struct server {
 	char serverhost[HOSTLEN + 1];
 	int serverport;
@@ -473,7 +507,7 @@ struct server {
 	int protected_ircops;
 	int used;
 	int connected;
-	int registered;
+	struct registered registered;
 	int stoned;
 	int linenoise;
 	int rx;
@@ -503,10 +537,12 @@ struct channel {
 	int allhostsknown;
 	time_t rejoin_at;
 	time_t last_quote;
+	time_t join_ts;
 	time_t userts;
 	struct join *joins;
 	struct part *parts;
 	struct pubmsg *pubmsgs;
+	struct nickchange *nickchanges;
 	struct locked locked;
         struct floodvars floodvars;
         struct banvars banvars;
@@ -523,8 +559,11 @@ struct channel {
 
 struct state {
 	char config[TINYSTRINGLEN + 1];
-	char nick[NICKLEN + 1];
-	char altnick[NICKLEN + 1];
+	/* this is our current nick, might be temporarily set
+	to getnick when attempting to change nick */
+        char nick[NICKLEN + 1];
+        char altnick[NICKLEN + 1];
+        char getnick[NICKLEN + 1];
         char user[USERLEN + 1];
         char host[HOSTLEN + 1];
 	char mode[CHANLEN + 1];
@@ -538,7 +577,7 @@ struct state {
 	char state[STRINGLEN + 1];
 	int customuser;
 	int vhost;
-	int use_alt_nick;
+	int use_altnick;
 	int sockfd;
 	int loopforever;
 	int newnick;
@@ -548,11 +587,12 @@ struct state {
 	int quitting;
 	int reconnect;
 	time_t now;
-	time_t last_actions;
 	time_t startup;
 	time_t postnj_checks_at;
 	time_t admints;
 	time_t last_linenoise;
+	time_t last_gn;
+	struct qauth qauth;
 	struct authed *autheds;
 	struct botuser *admins;
 	struct channel *channels;
@@ -562,4 +602,5 @@ struct state {
 	struct server *current_server;
 	struct timed *timeds;
         struct mysqldb mysqldb;
+	struct last_action last_action;
 };
