@@ -268,21 +268,16 @@ int main(int argc, char *argv[])
 			break;
 		}
 		/* do some housekeeping */
-		processenv();	
-		/* check if it's safe to do some server actions */
-                if(mystate->now <= mystate->last_actions) {
-                        continue;
+                processenv();
+                /* convert all mmodes to timeds */
+                mmodes2timeds();
+                /* process timeds */
+                switch(processtimeds()) {
+                case OER_PROCESSTIMEDS_ERR_WRITE:
+                        mystate->reconnect = OER_RECONNECT_ERROR;
+                        reconnect();
+                        break;
                 }
-		/* convert all mmodes to timeds */
-		mmodes2timeds();
-		/* process timeds */
-		switch(processtimeds()) {
-		case OER_PROCESSTIMEDS_ERR_WRITE:
-			mystate->reconnect = OER_RECONNECT_ERROR;
-			reconnect();
-			break;
-		}
-		mystate->last_actions = mystate->now;
 	}
         mysql_close(&mystate->mysqldb.mysqldbconn);
 	exit(EXIT_SUCCESS);
