@@ -261,16 +261,39 @@ if [ "$CREATECONF" = "y" ]; then
 	
     readln "Please specify bot admin handle [EQU] " "EQU"
     ADMIN=$ans
-    readln "Please specify bot admin flags [n] " "n"
+    readln "Please specify bot admin flags (n = unremovable, d = dynamic) [n] " "n"
     ADMINFLAGS=$ans
+    ISDYN=`$ECHO "$ADMINFLAGS" | sed 's/[0-9A-Za-ce-z]//g'`
+    if [ "$ISDYN" = "d" ]; then
+        stty -$ECHO
+        readln "Please specify your password [] " ""
+        PW=$ans
+        stty $ECHO
+        $ECHO ""
+        $ECHO ""
+        $ECHO -n "Crypting password ... "
+        CRYPT=`./mycrypt $PW`
+
+        if [ $RC -eq 1 ]; then
+                $ECHO "failed"
+                $ECHO ""
+            exit 1
+        fi
+
+        $ECHO "ok"
+        $ECHO ""
+    fi
     
     INSERT_ADMINS="INSERT INTO admins VALUES ('$ADMIN','$ADMINFLAGS','$HANDLE')"
 
     readln "Please specify bot admin hostmask [equ@*.equnet.org] " "equ@*.equnet.org"
     ADMINHOSTMASK=$ans
-    
     INSERT_HOSTMASKS="INSERT INTO hostmasks VALUES ('$ADMINHOSTMASK','',1,'$ADMIN','$HANDLE')"
-	
+
+    if [ "$ISDYN" = "d" ]; then
+	INSERT_PASSWORDS="INSERT INTO passwords VALUES ('$ADMIN','',2,'$CRYPT','$HANDLE')"
+    fi
+
     readln "Please specify default IRC server to connect to [irc.equnet.org] " "irc.equnet.org"
     SERVER=$ans
     readln "Please specify port [6667] " "6667"
@@ -291,7 +314,7 @@ if [ "$CREATECONF" = "y" ]; then
     $ECHO ""
     $ECHO -n "Importing data into database... "
 
-    for i in 1 2 3 4; do
+    for i in 1 2 3 4 5; do
 	case $i in
 	    1)
 	    INSERT_COMMAND=$INSERT_CONF
@@ -305,8 +328,15 @@ if [ "$CREATECONF" = "y" ]; then
 	    4)
 	    INSERT_COMMAND=$INSERT_SERVERS
 	    ;;
+	    5)
+	    INSERT_COMMAND=$INSERT_PASSWORDS
+	    ;;
 	esac
 
+	if [ $i = "5" -a "$ISDYN" != "d" ]; then
+	    continue
+	fi
+
 	if [ "$DBUSERPW" = "" ]; then
 	    LOGTHIS=`( $ECHO "$INSERT_COMMAND" | $1/mysql -h $HOST -u $DBUSER $DB ) 2>&1`
 	    RC=$?
