@@ -80,6 +80,9 @@ CREATE TABLE conf (
   prefix CHAR(32) NOT NULL DEFAULT '',
   realname VARCHAR(64) NOT NULL DEFAULT '',
   vhost VARCHAR(63) DEFAULT '',
+  qhost VARCHAR(80) DEFAULT '',
+  qname VARCHAR(32) DEFAULT '',
+  qpassword VARCHAR(32) DEFAULT '',
   ident VARCHAR(32) NOT NULL DEFAULT '',
   PRIMARY KEY (ident)
 ) TYPE=MyISAM PACK_KEYS=1;
@@ -95,6 +98,8 @@ CREATE TABLE floodvars (
   f_interval SMALLINT UNSIGNED NOT NULL DEFAULT '60',
   f_lines SMALLINT UNSIGNED NOT NULL DEFAULT '10',
   f_chars SMALLINT UNSIGNED NOT NULL DEFAULT '1600',
+  f_nickflood_expire SMALLINT UNSIGNED NOT NULL DEFAULT '7200',
+  f_nickflood_changes SMALLINT UNSIGNED NOT NULL DEFAULT '5',
   ident VARCHAR(64) NOT NULL DEFAULT '',
   PRIMARY KEY (channel,ident)
 ) TYPE=MyISAM PACK_KEYS=1;
